﻿using System;
using System.IO;
using System.Xml;
using System.Xml.Xsl;
using System.Collections.Generic;
using System.Text;
using Dynamicweb.Rendering;
using Dynamicweb.Ecommerce.Orders;
using Dynamicweb.Modules.UserManagement;
using System.Xml.XPath;
using Dynamicweb.Extensibility;
using Dynamicweb.Security.UserManagement;
using Dynamicweb.Environment;
using Dynamicweb.Extensibility.Notifications;

namespace Dynamicweb.Ecommerce.LiveIntegration
{
    internal enum SortDirection
    {
        Asc = 0,
        Desc
    }

    internal class IntegrationCustomerCenterHandler
    {
        private static readonly string OrderLinesLoop = "OrderLinesLoop";

        public static Template RetrieveItemDetailsFromRemoteSystem(Template template, string callType, User user, string itemID)
        {
            if (!string.IsNullOrEmpty(itemID) && !string.IsNullOrEmpty(Context.Current.Request.GetString("Redirect")) && Context.Current.Request.GetBoolean("addToCart"))
            {
                HandleReorder(itemID);
            }
            else
            {
                if (!string.IsNullOrEmpty(callType) && user != null)
                {
                    string request = GetItemDetailsRequest(callType, Helpers.CustomerIdentifier, itemID);
                    Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Request RetrieveItemDetailsFromRemoteSystem sent: '{0}'.", request));
                    XmlDocument response = Connector.RetrieveDataFromRequestString(request);
                    if (response != null && !string.IsNullOrEmpty(response.InnerXml))
                    {
                        Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Response RetrieveItemDetailsFromRemoteSystem received: '{0}'.", response.InnerXml));
                        if (template.Type != Template.TemplateType.Xslt)
                        {
                            ProcessItemDetailsResponse(response, template, callType);
                        }
                        else
                        {
                            MakeXslTransformation(response, template);
                        }
                    }
                    else
                    {
                        Logger.Instance.Log(ErrorLevel.ResponseError, "Response RetrieveItemDetailsFromRemoteSystem returned null.");
                    }
                }
            }
            return template;
        }

        public static Template RetrieveItemsListFromRemoteSystem(Template template, string callType, User user, int pageSize, int pageIndex, out int totalItemsCount)
        {
            totalItemsCount = 0;
            if (!string.IsNullOrEmpty(callType) && user != null)
            {
                string request = GetRequest(callType, true, Helpers.CustomerIdentifier, null, pageSize, pageIndex);
                Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Request RetrieveItemsListFromRemoteSystem sent: '{0}'.", request));
                XmlDocument response = Connector.RetrieveDataFromRequestString(request);
                if (response != null && !string.IsNullOrEmpty(response.InnerXml))
                {
                    Logger.Instance.Log(ErrorLevel.DebugInfo, string.Format("Response RetrieveItemsListFromRemoteSystem received: '{0}'.", response.InnerXml));
                    if (template.Type != Template.TemplateType.Xslt)
                    {
                        ProcessItemsListResponse(response, template, callType, pageSize, pageIndex, out totalItemsCount);
                    }
                    else
                    {
                        MakeXslTransformation(response, template);
                    }
                }
                else
                {
                    Logger.Instance.Log(ErrorLevel.ResponseError, "Response RetrieveItemsListFromRemoteSystem returned null.");
                }
            }
            return template;
        }

        private static string GetItemDetailsRequest(string callType, string userID, string itemID)
        {
            return GetRequest(callType, false, userID, itemID, 0, 0);
        }

        private static string GetRequest(string callType, bool isList, string userID, string itemID, int pageSize, int pageIndex)
        {
            string ret = string.Format("type=\"{0}\" customerID=\"{1}\"", callType, userID);
            if (isList)
            {
                int firstItem = pageSize * pageIndex - pageSize + 1;
                if (firstItem <= 0)
                    firstItem = 1;

                string sorting = string.Empty;
                if (!string.IsNullOrEmpty(Context.Current.Request.GetString("ICCSortByField")))
                {
                    SortDirection sortDirection = SortDirection.Asc;
                    Enum.TryParse<SortDirection>(Context.Current.Request.GetString("ICCSortByDirection"), out sortDirection);
                    sorting = string.Format(" sortByField=\"{0}\" sortDirection=\"{1}\"", Context.Current.Request.GetString("ICCSortByField"), sortDirection.ToString());
                }

                string search = string.Empty;
                if (!string.IsNullOrEmpty(Context.Current.Request.GetString("ICCSearchField")) && !string.IsNullOrEmpty(Context.Current.Request.GetString("ICCSearchValue")))
                {
                    search = string.Format(" searchField=\"{0}\" searchValue=\"{1}\"", Context.Current.Request.GetString("ICCSearchField"), Context.Current.Request.GetString("ICCSearchValue"));
                }

                var queryParameters = new StringBuilder();
                foreach (string key in System.Web.HttpContext.Current.Request.QueryString.AllKeys)
                {

                    if (string.IsNullOrEmpty(Context.Current.Request.GetString(key)))
                        continue;

                    if (key.ToLower().StartsWith("icc_"))
                    {
                        queryParameters.AppendFormat(" {0}=\"{1}\"", key.Substring(4), Context.Current.Request.GetString(key));
                    }
                }

                foreach (string key in System.Web.HttpContext.Current.Request.Form.AllKeys)
                {

                    if (string.IsNullOrEmpty(Context.Current.Request.GetString(key)))
                        continue;

                    if (key.ToLower().StartsWith("icc_"))
                    {
                        queryParameters.AppendFormat(" {0}=\"{1}\"", key.Substring(4), Context.Current.Request.GetString(key));
                    }
                }

                ret = string.Format("<GetList {0} requestAmount=\"{1}\" firstItem=\"{2}\"{3}{4}{5}></GetList>", ret, pageSize, firstItem, sorting, search, queryParameters.ToString());

            }
            else if (!string.IsNullOrEmpty(itemID))
            {
                ret = string.Format("<GetItem {0}  documentNO=\"{1}\"></GetItem>", ret, itemID);
            }
            return ret;
        }

        private static void ProcessItemDetailsResponse(XmlDocument response, Template template, string callType)
        {
            try
            {
                XmlNode orderNode = response.SelectSingleNode("//item [@table='EcomOrders']");
                //Set Order tags
                if (orderNode != null && orderNode.ChildNodes.Count > 0)
                {
                    foreach (XmlNode itemProperty in orderNode.ChildNodes)
                    {
                        if (itemProperty.Attributes["columnName"] != null)
                        {
                            string tagName = itemProperty.Attributes["columnName"].Value;
                            template.SetTag(tagName, itemProperty.InnerText);
                        }
                    }
                }
                //Set OrderLines loop
                XmlNodeList orderLinesNodes = response.SelectNodes("//item [@table='EcomOrderLines']");
                if (orderLinesNodes != null && orderLinesNodes.Count > 0)//Process OrderLines
                {
                    Template orderLinesLoop = template.GetLoop(OrderLinesLoop);

                    foreach (XmlNode orderLineNode in orderLinesNodes)
                    {
                        foreach (XmlNode itemProperty in orderLineNode.ChildNodes)
                        {
                            if (itemProperty.Attributes["columnName"] != null)
                            {
                                string tagName = itemProperty.Attributes["columnName"].Value;
                                orderLinesLoop.SetTag(tagName, itemProperty.InnerText);
                            }
                        }
                        orderLinesLoop.CommitLoop();
                    }
                }
                SetUrlTags(template);
            }
            catch (Exception e)
            {
                Logger.Instance.Log(ErrorLevel.Error, string.Format("Response does not match schema: '{0}'.", e.Message));
            }
        }

        private static void ProcessItemsListResponse(XmlDocument response, Template template, string callType, int pageSize, int pageIndex, out int totalItemsCount)
        {
            totalItemsCount = 0;
            string tagPrefix = string.Format("Ecom:IntegrationCustomerCenter.{0}", callType);
            try
            {
                XmlNodeList items = response.SelectNodes("//item");

                if (items != null && items.Count > 0)
                {
                    XmlNode itemsNode = response.SelectSingleNode("Items");
                    if (itemsNode != null && itemsNode.Attributes["totalCount"] != null)
                    {
                        Int32.TryParse(itemsNode.Attributes["totalCount"].Value, out totalItemsCount);
                    }
                    if (totalItemsCount <= 0)
                    {
                        totalItemsCount = items.Count;
                    }

                    string itemsLoopName = string.Format("{0}Loop", tagPrefix);
                    Template itemsLoop = template.GetLoop(itemsLoopName);

                    foreach (XmlNode item in items)
                    {
                        foreach (XmlNode itemProperty in item.ChildNodes)
                        {
                            if (itemProperty.Attributes["columnName"] != null)
                            {
                                string tagName = itemProperty.Attributes["columnName"].Value;
                                itemsLoop.SetTag(tagName, itemProperty.InnerText);
                            }
                        }
                        SetUrlTags(itemsLoop);
                        itemsLoop.CommitLoop();
                    }
                }
                else
                {
                    //set empty list tag
                    template.SetTag(string.Format("{0}.EmptyList", tagPrefix), true);
                }
            }
            catch (Exception e)
            {
                Logger.Instance.Log(ErrorLevel.Error, string.Format("Response does not match schema: '{0}'.", e.Message));
            }
        }

        private static void MakeXslTransformation(XmlDocument doc, Template template)
        {
            try
            {
                XslCompiledTransform xslTransform = new XslCompiledTransform();
                StringWriter writer = new StringWriter();
                xslTransform.Load(new XPathDocument(new StringReader(template.Html)));
                xslTransform.Transform(doc.CreateNavigator(), null, writer);
                template.Html = writer.ToString();
            }
            catch (Exception ex)
            {
                string msg = string.Format("Error in xsl transformation: '{0}'.", ex.Message);
                template.Html = msg;
                Logger.Instance.Log(ErrorLevel.Error, msg);
            }
        }

        private static void SetUrlTags(Template template)
        {
            string userID = User.GetCurrentUser() != null ? User.GetCurrentUser().ID.ToString() : string.Empty;
            string orderID = Ecommerce.Common.Context.Cart != null ? Ecommerce.Common.Context.Cart.Id : string.Empty;
            string shopID = string.Empty;
            if (Dynamicweb.Frontend.PageView.Current() != null && Dynamicweb.Frontend.PageView.Current().Area != null)
            {
                shopID = (string)Dynamicweb.Frontend.PageView.Current().Area.EcomShopId;
            }
            template.SetTag("UrlUserID", userID);
            template.SetTag("UrlOrderID", orderID);
            template.SetTag("UrlShopID", shopID);
        }

        private static void HandleReorder(string itemID)
        {
            Order order = Order.GetOrderById(itemID);
            if (order != null)
            {
                Order ecomCart = null;
                if (!Frontend.Cart.CartCatch.CartIsPresent())
                {
                    ecomCart = new Order();
                    ecomCart.IsCart = true;
                    ecomCart.LanguageId = Common.Context.LanguageID;                    
                    Common.Context.SetCart(ecomCart);
                }

                ecomCart = Common.Context.Cart;

                foreach (OrderLine orderLineToCopy in order.OrderLines)
                {
                    var OrderlineBuilder = new OrderLineBuilderConfig()
                    {
                        ProductId = orderLineToCopy.ProductId,
                        Quantity = orderLineToCopy.Quantity,
                        VariantId = orderLineToCopy.ProductVariantId,
                        VariantText = orderLineToCopy.ProductVariantText,
                        UnitId = orderLineToCopy.UnitId,
                        OrderLineType = orderLineToCopy.GetOrderLineType(),
                        ReferenceUrl = orderLineToCopy.Reference,
                        PageId = orderLineToCopy.PageId,
                        ProductName = string.IsNullOrEmpty(orderLineToCopy.ProductId) ? orderLineToCopy.ProductName : string.Empty,
                        ProductNumber = string.IsNullOrEmpty(orderLineToCopy.ProductId) ? orderLineToCopy.ProductNumber : string.Empty,
                        OrderLineFieldValues = orderLineToCopy.OrderLineFieldValues,
                    };


                    var newOrderLine = ecomCart.OrderLineBuilder(OrderlineBuilder);

                    
                    Frontend.Cart.CartCatch.SaveCart();
                    
                    //Prepare Orderline prices.                    
                    Dictionary<Products.Product, double> prods = new Dictionary<Products.Product, double>();
                    foreach (OrderLine ol in ecomCart.OrderLines)
                    {
                        if (ol.HasType(OrderLineType.Product))
                        {
                            if (!prods.ContainsKey(ol.Product))
                            {
                                prods.Add(ol.Product, ol.Quantity);
                            }
                        }
                    }
                    Prices.PriceManager.PreparePrices(prods);
                    
                    NotificationManager.Notify(Notifications.Ecommerce.Cart.Line.Added, new Notifications.Ecommerce.Cart.Line.AddedArgs(newOrderLine, ecomCart));
                }
                ecomCart.Comment = order.Comment;
                ecomCart.CustomerComment = order.CustomerComment;
                OrderFieldValueCollection orderFieldValues = new OrderFieldValueCollection();
                foreach (OrderFieldValue orderField in order.OrderFieldValues)
                {
                    orderFieldValues.Add(new OrderFieldValue(orderField.OrderField, orderField.Value));
                }
                ecomCart.OrderFieldValues = orderFieldValues;
                ecomCart.Save();                

                //Set last cart to this order 
//                Frontend.Cart.CartCatch.SaveCart(order);

                //Remove non-active and non-existing products
                ecomCart.RemoveNoneActiveProducts();

                string redirectUrl = Context.Current.Request.GetString("Redirect");
                if (Context.Current.Request.GetInt32("Redirect") != 0)
                {
                    redirectUrl = "/Default.aspx?ID=" + Context.Current.Request.GetString("Redirect");
                }
                System.Web.HttpContext.Current.Response.Redirect(redirectUrl);
                System.Web.HttpContext.Current.Response.End();
            }
            else
            {
                Logger.Instance.Log(ErrorLevel.Error, string.Format("Can not find order with ID = '{0}'", itemID));
            }
        }
    }
}
